
/* Program to load a wave file and loop playing it using SDL sound */

/* loopwaves.c is much more robust in handling WAVE files -- 
	This is only for simple WAVEs
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "mydebug.h"

#include "SDL.h"
#include "SDL_audio.h"

struct {
	SDL_AudioSpec spec;
	Uint8   *sound;			/* Pointer to wave data */
	Uint32   soundlen;		/* Length of wave data */
	int      soundpos;		/* Current play position */
} wave;

int done;

void fillerup(void *unused, Uint8 *stream, int len)
{
	Uint8 *waveptr;
	int    wavetoplay;

	if( (wave.soundlen-wave.soundpos)<=0 )
	{
		D(bug("Playing finito!\n"));
		done=1;
		return;
	}

	/* Set up the pointers */
	waveptr = wave.sound + wave.soundpos;

	wavetoplay = (wave.soundlen - wave.soundpos)>len ? len : (wave.soundlen - wave.soundpos);

	/* Go! */
	D(bug("Mixxo %ld bytes, da %lx...\n",wavetoplay,waveptr));
	SDL_MixAudio(stream, waveptr, wavetoplay, SDL_MIX_MAXVOLUME);

	wave.soundpos += wavetoplay;
}

main(int argc, char *argv[])
{
	/* Load the SDL library */
	if ( SDL_Init(SDL_INIT_AUDIO) < 0 ) {
		fprintf(stderr, "Couldn't initialize SDL: %s\n",SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

	if ( argv[1] == NULL ) {
		fprintf(stderr, "Usage: %s <wavefile>\n", argv[0]);
		exit(1);
	}

	/* Load the wave file into memory */
	if ( SDL_LoadWAV(argv[1],
			&wave.spec, &wave.sound, &wave.soundlen) == NULL ) {
		fprintf(stderr, "Couldn't load %s: %s\n",
						argv[1], SDL_GetError());
		exit(1);
	}

	D(bug("Suono wav: len:%ld data:%lx T:%lx Fq:%ld C:%d\n",wave.soundlen,wave.sound,wave.spec.format,wave.spec.freq,wave.spec.channels));
	wave.spec.callback = fillerup;

	/* Initialize fillerup() variables */
	if ( SDL_OpenAudio(&wave.spec, NULL) < 0 ) {
		fprintf(stderr, "Couldn't open audio: %s\n", SDL_GetError());
		SDL_FreeWAV(wave.sound);
		exit(2);
	}

	D(bug("Avvio il playback...\n"));
	SDL_PauseAudio(0);

	/* Let the audio run */
	while ( ! done && (SDL_GetAudioStatus() == SDL_AUDIO_PLAYING) )
		SDL_Delay(1000);

	/* Clean up on signal */
	SDL_CloseAudio();
	SDL_FreeWAV(wave.sound);
	exit(0);
}
